# define Mercurial extension metadata for evolution
#
# Copyright 2017 Pierre-Yves David <pierre-yves.david@ens-lyon.org>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.

__version__ = '6.0.0'
testedwith = '3.8.4 3.9.2 4.0.2 4.1.1'
minimumhgversion = '3.8'
buglink = 'https://bz.mercurial-scm.org/'
